import _extends from "@babel/runtime/helpers/esm/extends";
import React, { useCallback } from 'react';
import { field, createComponentMemo } from '../prop-types';
import MoleculeSwitch from '@s-ui/react-atom-switch';

var Switch = function Switch(_ref) {
  var switchField = _ref.switchField,
      tabIndex = _ref.tabIndex,
      onChange = _ref.onChange,
      errors = _ref.errors,
      alerts = _ref.alerts,
      renderer = _ref.renderer;
  var errorMessages = errors[switchField.id];
  var alertMessages = alerts[switchField.id];
  var onChangeCallback = useCallback(function (value) {
    return onChange(switchField.id, JSON.stringify(value));
  }, [onChange, switchField]);

  var switchProps = _extends({
    name: switchField.id,
    label: switchField.label,
    labelLeft: 'NOP',
    labelRight: 'ZIP',
    value: JSON.parse(switchField.value || 'false'),
    onToggle: onChangeCallback,
    type: 'single'
  }, switchField.disabled && {
    disabled: true
  }, switchField.hidden && {
    hidden: true
  }, !!errorMessages && {
    errorText: errorMessages.join('\n')
  }, !!alertMessages && {
    alertText: alertMessages.join('\n')
  });

  if (switchProps.hidden) {
    return null;
  }

  var rendererResponse = renderer({
    id: switchField.id,
    innerProps: switchProps
  }); // render custom component

  if ( /*#__PURE__*/React.isValidElement(rendererResponse)) return rendererResponse; // render SUI component

  return /*#__PURE__*/React.createElement("div", {
    className: "sui-FormBuilder-field sui-FormBuilder-Switch sui-FormBuilder-" + (switchProps.id || tabIndex)
  }, /*#__PURE__*/React.createElement(MoleculeSwitch, Object.assign({}, switchProps, rendererResponse)));
};

Switch.displayName = 'Switch';
export default /*#__PURE__*/React.memo(Switch, createComponentMemo('switchField'));