import _regeneratorRuntime from "@babel/runtime/regenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import React, { useState, useEffect, useCallback, useRef } from 'react';
import { json } from './prop-types';
import { reducer } from './reducer';
import { RULES } from './reducer/constants';
import { pickFieldById, fieldsToObject, fieldsNamesInOrderOfDefinition, changeFieldById } from './reducer/fields';
import ProxyField from './ProxyField';
import { inputSizes as fieldSizes } from '@s-ui/react-atom-input';
import AtomSpinner, { AtomSpinnerTypes } from '@s-ui/react-atom-spinner';

var FormBuilder = function FormBuilder(_ref) {
  var json = _ref.json,
      initFields = _ref.initFields,
      onChange = _ref.onChange,
      onBlur = _ref.onBlur,
      responseInterceptor = _ref.responseInterceptor,
      requestInterceptor = _ref.requestInterceptor,
      loader = _ref.loader,
      fieldSize = _ref.fieldSize,
      errors = _ref.errors,
      alerts = _ref.alerts,
      transformations = _ref.transformations,
      locale = _ref.locale,
      renderer = _ref.children;
  var _json$form = json.form,
      _json$form$fields = _json$form.fields,
      fields = _json$form$fields === void 0 ? [] : _json$form$fields,
      _json$form$rules = _json$form.rules,
      rules = _json$form$rules === void 0 ? {} : _json$form$rules,
      formID = _json$form.id;

  var _useState = useState(fields),
      stateFields = _useState[0],
      setStateFields = _useState[1];

  var _useState2 = useState(false),
      stateShowSpinner = _useState2[0],
      setStateShowSpinner = _useState2[1];

  var __PERFORMANCE_UGLY_HACK_STATE_FIELDS__ = useRef();

  __PERFORMANCE_UGLY_HACK_STATE_FIELDS__.current = stateFields;

  var changeField = function changeField(fieldId, fieldValue) {
    var nextFields = changeFieldById(__PERFORMANCE_UGLY_HACK_STATE_FIELDS__.current, fieldId, {
      value: fieldValue
    });
    setStateFields(nextFields);
    return nextFields;
  };

  var handlerChange = useCallback( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(id, value) {
      var reducerWithRules, timerShowSpinner, transformedValue, nextFields, nextStateFields;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              reducerWithRules = reducer(rules, formID, responseInterceptor, requestInterceptor, locale);
              timerShowSpinner = setTimeout(function () {
                return setStateShowSpinner(true);
              }, FormBuilder.USER_MINIMAL_DELAY);
              transformedValue = transformations(id, value);
              nextFields = changeField(id, transformedValue);
              clearTimeout(timerShowSpinner);
              _context.next = 7;
              return reducerWithRules(nextFields, {
                type: RULES,
                id: id
              });

            case 7:
              nextStateFields = _context.sent;
              clearTimeout(timerShowSpinner);
              setStateFields(nextStateFields);
              onChange(_extends({}, fieldsToObject(nextStateFields), {
                __FIELD_CHANGED__: id
              }));
              setStateShowSpinner(false);

            case 12:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }(), []); // eslint-disable-line

  useEffect(function () {
    var reducerWithRules = reducer(rules, formID, responseInterceptor, requestInterceptor, locale);
    var timerShowSpinner = setTimeout(function () {
      return setStateShowSpinner(true);
    }, FormBuilder.USER_MINIMAL_DELAY);
    fieldsNamesInOrderOfDefinition(fields).reduce( /*#__PURE__*/function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(previousPromise, fieldId) {
        var previousFields, nextFieldsChanged, nextFieldsWithRules;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return previousPromise;

              case 2:
                previousFields = _context2.sent;

                if (!(!initFields[fieldId] || initFields[fieldId] === '')) {
                  _context2.next = 5;
                  break;
                }

                return _context2.abrupt("return", previousFields);

              case 5:
                nextFieldsChanged = changeField(fieldId, initFields[fieldId]);
                _context2.next = 8;
                return reducerWithRules(nextFieldsChanged, {
                  type: RULES,
                  id: fieldId
                });

              case 8:
                nextFieldsWithRules = _context2.sent;
                return _context2.abrupt("return", nextFieldsWithRules);

              case 10:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      return function (_x3, _x4) {
        return _ref3.apply(this, arguments);
      };
    }(), Promise.resolve(__PERFORMANCE_UGLY_HACK_STATE_FIELDS__.current)).then(function (nextFields) {
      clearTimeout(timerShowSpinner);
      setStateFields(nextFields);
      setStateShowSpinner(false);
    });
  }, []); // eslint-disable-line

  return /*#__PURE__*/React.createElement("div", {
    className: "sui-FormBuilder"
  }, stateFields.map(function (field, index) {
    return /*#__PURE__*/React.createElement(ProxyField, {
      key: field.id,
      field: field,
      tabIndex: index + 1,
      onChange: handlerChange,
      onBlur: onBlur,
      fieldSize: fieldSize,
      errors: errors,
      alerts: alerts,
      renderer: renderer
    });
  }), stateShowSpinner && /*#__PURE__*/React.createElement(AtomSpinner, {
    type: AtomSpinnerTypes.SECTION,
    loader: loader
  }));
};

FormBuilder.USER_MINIMAL_DELAY = 250;
FormBuilder.displayName = 'FormBuilder';
FormBuilder.defaultProps = {
  initFields: {},
  onChange: function onChange() {},
  onBlur: function onBlur() {},
  responseInterceptor: function responseInterceptor(_ref4) {
    var response = _ref4.response;
    return response;
  },
  requestInterceptor: function requestInterceptor() {},
  errors: {},
  alerts: {},
  transformations: function transformations(_, value) {
    return value;
  },
  locale: 'es-ES',
  children: function children() {
    return {};
  }
};
export { fieldSizes as formBuilderFieldSizes };
export { pickFieldById as formBuilderPickFieldById };
export { changeFieldById as formBuilderChangeFieldById };
export { fieldsNamesInOrderOfDefinition as formBuilderFieldsNamesInOrderOfDefinition };
export { checkConstraintsFactory } from './Standard';
export default FormBuilder;